<?php

namespace QUI\RabbitMQServer\REST;

use QUI;
use QUI\RabbitMQServer\REST\Route\LogFailedTask;
use QUI\RabbitMQServer\REST\Route\RequeueTask;
use QUI\RabbitMQServer\REST\Route\WriteServerLogEntry;
use QUI\RabbitMQServer\REST\Route\WriteTaskLog;
use QUI\REST\ProviderInterface;
use QUI\REST\Server;
use Slim\Interfaces\RouteCollectorProxyInterface;

use function is_null;

class Provider implements ProviderInterface
{
    public function register(Server $server): void
    {
        $slim = $server->getSlim();

        $slim->group('/rabbitmq', function (RouteCollectorProxyInterface $routeCollector) {
            $routeCollector->group('/v1', function (RouteCollectorProxyInterface $routeCollector) {
                $routeCollector->group('/task', function (RouteCollectorProxyInterface $routeCollector) {
                    $routeCollector->post('/writeLog', WriteTaskLog::class);
                    $routeCollector->post('/requeue', RequeueTask::class);
                    $routeCollector->post('/logFailure', LogFailedTask::class);
                });

                $routeCollector->post('/writeServerLogEntry', WriteServerLogEntry::class);
            });
        });
    }

    public function getOpenApiDefinitionFile(): bool|string
    {
        // TODO
        return false;
    }

    public function getTitle(QUI\Locale $Locale = null): string
    {
        if (is_null($Locale)) {
            $Locale = QUI::getLocale();
        }

        return $Locale->get('quiqqer/rabbitmqserver', 'REST.provider_title');
    }

    public function getName(): string
    {
        return 'quiqqer_rabbitmqserver';
    }
}
