<?php

namespace QUI\RabbitMQServer\REST\Route;

use PCSG\Tasks\TaskSerializerInterface;
use Psr\Http\Message\ResponseInterface as ResponseInterface;
use QUI\Exception;
use QUI\QueueManager\Interfaces\QueueJobRepositoryInterface;
use QUI\QueueManager\JobStatus;
use QUI\QueueManager\Repository\QueueJobRepository;
use QUI\RabbitMQServer\Interface\RabbitMQProviderInterface;
use QUI\RabbitMQServer\Interface\TaskSchedulerInterface;
use QUI\RabbitMQServer\Provider;
use QUI\REST\Utils\RequestUtils;

use function is_null;

/**
 * Log a task that failed and could not be completed.
 *
 * This will put the task data in the quiqqer/queuemanager queue job database for further inspection.
 */
class LogFailedTask extends AbstractRoute
{
    private TaskSerializerInterface $taskSerializer;
    private TaskSchedulerInterface $taskScheduler;

    /**
     * @param QueueJobRepositoryInterface|null $jobRepository
     * @param RabbitMQProviderInterface|null $rabbitMQProvider
     * @throws Exception
     */
    public function __construct(
        private ?QueueJobRepositoryInterface $jobRepository = null,
        private ?RabbitMQProviderInterface $rabbitMQProvider = null
    ) {
        if (is_null($this->jobRepository)) {
            $this->jobRepository = new QueueJobRepository();
        }

        if (is_null($this->rabbitMQProvider)) {
            $this->rabbitMQProvider = new Provider();
        }

        $this->taskSerializer = $this->rabbitMQProvider->getTaskSerializer();
        $this->taskScheduler = $this->rabbitMQProvider->getTaskScheduler();
    }

    protected function handleRequest(): ResponseInterface
    {
        try {
            $serializedWorkerTask = RequestUtils::getFieldFromRequest($this->request, 'task');
            $task = $this->taskSerializer->deserializeWorkerTaskFromQueue($serializedWorkerTask);

            $queueJob = $this->taskScheduler->createQueueJobFromWorkerTask($task);
            $queueJob->status = JobStatus::MANUAL_CHECK_REQUIRED;

            $this->jobRepository->insert($queueJob);
        } catch (\Exception $exception) {
            return $this->buildExceptionErrorResponse($exception);
        }

        return $this->buildSuccessResponse('Task written to db as queue job.');
    }
}
